function V_STR2NUM, svalue, TYPE = type, SILENT = silent, IN_TYPE = in_type

;+ $Id: v_str2num.pro,v 1.5 2007/02/20 16:15:34 flo Exp $
;
; NAME:
;          V_STR2NUM
;
; PURPOSE:
;           Return the numeric value of a string, if possible.
;           Otherwise return the input string.
;           Return smallest Type that can accommodate the value, except byte.
;
; CALLING SEQUENCE:
;          result = V_STR2NUM(svalue, [Type=Type])
;
; INPUT:
;          svalue = a scalar string to be converted to its numeric value 
;               or a numeric value to be converted to its 'smallest' form (except byte)
;
; OUTPUT:
;          result = numeric value of input string, or unaltered string
;               if numeric conversion is not possible.
;              (smallest Type that can accommodate it, except byte).
;
; OPTIONAL KEYWORD:
;          TYPE:  output, contain the IDL type of the result
;          IN_TYPE: input, contains the desired IDL type of result.
;          (1 to 7 or 14 - this may result in round-off errors).
;          This option allows convertion to byte. Beware that, if later converted 
;          back to string, this can yield major problems in the IDL session 
;          (if ascii < 32).
;
;	SILENT - Disable warning messages.
;
; EXAMPLES:
;
;          IDL> x = v_str2num('123')     ; Returns '123' in a numerical variable
;          IDL> x = v_str2num(123L)     ; Converts '123' to short integer
;
; PROCEDURE / LIMITATIONS:
;          The input string is first tested to see if it is an ISO time string
;          by searching for ':' or 'T' (see modification history). If so it
;          is returned unchanged as a string. 
;          Dates/time strings can be further handled with V_TIME.PRO
;          Also filters S/C clock time strings by searching for '/'.
;          S/C clock time strings can be further handled with V_SCET2UT.PRO
;
;         The string is then tested for a complex value: OK if contains two 
;          numerical values separated by '(', ',' or ')' (minimal testing). 
;          Does not support double precision complexes.
;
;          Real Types are identified by '.', 'E' or 'D' only.
;          Integer Types are then tested for the optimal format 
;          (from INTEGER to LONG64 - conversion to bytes is removed to avoid 
;          messing with function characters in the 0-31 range: the result is 
;          converted back to string in higher level routines).
;          Converts to double precision if integer with more than 18 numerals,
;          or if floating point with more than 7 numerals (to retain accuracy).
;
;          Beware that conversion problems force the argument to be returned as a
;          string with no warning - result type should be tested.
;
;          Beware that a conversion error may occur before call when using 
;          a numerical argument. Example:
;          print, v_str2num(68630136.701), f='(F20.4)'  ; is rounded before call
;          print, 68630136.701, f='(F20.4)'     ; rounding by IDL itself
;
;
;
; HISTORY:
;          Original str2num.pro:
;          Written by John D. Koch, July, 1994
;          27 July 1999, M. Barker: fixed bug that converted a date in format of
;		1991-05-12 to 1991, so that if a '-' is 
;		detected and neither 'e' or 'E' are detected,
;		the value is left as a string.
;
;          Processing has been completely changed - S. Erard, LESIA, June 2005
;               + use "new" integer Types (long and long64)
;               + removed conversion to bytes, to avoid non printable characters
;               Maintained name and arguments for compatibility
;          SE, July 2005: handles resynchronization number in S/C clock counts
;          SE, Nov 2005: added forced type option (IN_TYPE) + SILENT option.
; 
;----------------------------------------------------------------------

 if ( N_PARAMS() NE 1)then begin
     print,'Syntax - result =v_str2num(svalue,[ Type = Type])
     return, -1
 endif 

 value = 0
 s = size(svalue)			
 if ( s(0) NE 0 ) and not(keyword_set(silent)) then begin
      message, 'Argument must be a scalar using first element ', /info
      svalue = svalue(0)
 endif
 Ttype = 7          ; default Type is string
If Keyword_set(In_Type) then $
     if in_type LT 1 or (in_Type GT 7 and In_Type NE 14) then in_type =0 ; unset force type


;	trap value as a string if may be a date/time expression
 if (strpos(svalue,':') GE 0) or (strpos(svalue,'T') GE 0) then goto, CONVERT
 if strpos((no_white=strcompress(svalue,/remove_all)),'-') GT 1 then begin
   if strpos(no_white,'e') EQ -1 and strpos(no_white,'E') EQ -1 $
   then goto, CONVERT
 endif
;	trap value as a string if it looks like a time plus a number of resynchronization
 if (strpos(svalue,'/') GE 0) then goto, CONVERT


 l = strlen(svalue)
 temp = svalue
 on_ioerror, CONVERT		; jump to CASE if conversion on next line fails


;  Complex Types only 

vect = strsplit(svalue, /ext, '(,)')          ; parse string
if size(vect, /dim) EQ 2 then begin          ; assumed complex if 2 elt found
   temp = complex(vect(0),vect(1))
   Ttype=6
   goto, CONVERT
 endif

;  Non-complex Types 

if (strpos(svalue,'.') GT 0) then begin       ; Floating Types
   temp = double(svalue)     ; will return a string on conversion error
   Ttype = 4   
   vect = strsplit(svalue, /ext, '.')                    ; parse string
   if total(strlen(vect)) GE 8 then Ttype = 5      ; double required

endif else begin                                             ; Integer Types
 
   temp = long64(svalue)     ; convert to largest integer Type
                                              ; or return a string on conversion error
    atemp = abs(temp)
   if (atemp GT 2UL^31-1) then Ttype = 14 $
       else if (atemp GT 32767) then Ttype = 3  else Ttype = 2
;   if (temp GE 0) and (temp LE 255) then Ttype = 1      ; intentional, see header
;   if (temp GE 0) and (temp GT 2UL^32-1) then Ttype = 15 
; (the above line does not work: convertion is already performed)

   if (strpos(svalue,'E') GT 0) then Ttype = 4      ; other floating notations
   if (strpos(svalue,'D') GT 0) then Ttype = 5
   if (strlen(svalue) GE 19) then Ttype = 5         ; double required
 endelse

 on_ioerror, NULL

 CONVERT:
; force output type only if convertible 
If Ttype NE 7 and Keyword_set(In_Type) then Ttype = In_Type 
	CASE Ttype OF
;                15 : value=Ulong64(temp)
                14 : value=long64(temp)
                7 : value=svalue
                6 : value=temp
                5 : value=double(svalue)
                4 : value=float(svalue)
                3 : value=long(temp)
                2 : value=fix(temp)
                1 : value=byte(temp)
	  else: if NOT keyword_set(silent) then message,'No corresponding type'
	ENDCASE

Type = Ttype
 return,value
 end

